import numpy as np
import matplotlib.pyplot as plt


def cumulative_bar(*arrays, linewidth=1, bar_color=None, ax=None, zorder=None):
    """
    Cumulative bar plot.

    Args:
        arrays (iterable): an iterable of lists/arrays.
        linewidth (int, optional): Width for the lines between bars and around the bars. Defaults to 2.
        bar_color (str or list, optional): Colors for the bars: Defaults to None.. You can either provide:
            - a single color as a string or array of RGB values.
            - a list containing the colors for the bars of each array.
            - a list of lists containing the color of every single bar.
        ax (matplotlib.axes, optional): matplotlib axis object to plot on. If not passed, one will be created.

    Returns:
        (tuple): figure, axis
    """

    if ax is not None:
        ax = ax
        fig = plt.gcf()
    else:
        fig, ax = plt.subplots()

    default_color = "k"

    if bar_color is None:
        bar_color = [[default_color] * len(arr) for arr in arrays]
    elif not isinstance(bar_color, list):
        bar_color = [[bar_color] * len(arr) for arr in arrays]
    elif (len(bar_color) == len(arrays)) and not isinstance(bar_color[0], list):
        bar_color = [[bar_color[i]] * len(arr) for i, arr in enumerate(arrays)]

    for data_i, data in enumerate(arrays):

        height = np.concatenate((np.array([data[0]]), np.diff(data)))
        bottom = np.concatenate((np.array([0]), np.array(data[:-1])))

        total_width = 0.8
        single_bar_widths = 0.7 * total_width / len(data)
        linelength = total_width / len(data) - single_bar_widths
        x = data_i + np.concatenate(
            (np.array([0]), np.arange(1, len(data)) * total_width / len(data))
        )
        left_edges = x - single_bar_widths / 2
        right_edges = x + single_bar_widths / 2

        ax.bar(
            x,
            height,
            bottom=bottom,
            width=single_bar_widths,
            edgecolor=default_color,
            color=bar_color[data_i],
            linewidth=linewidth,
            zorder=zorder,
        )

        for i in range(len(data) - 1):
            ax.plot(
                [left_edges[i + 1], right_edges[i]],
                [data[i], data[i]],
                c=default_color,
                lw=linewidth,
                zorder=zorder,
            )

    ax.set(xticks=np.arange(len(arrays)))

    return fig, ax


import os
import glob
import shutil
from pathlib import Path
from PIL import Image
from IPython import display
import matplotlib.pyplot as plt


class GifMaker:
    def __init__(self, filename, fps=30.0):
        """
        Saves gif from added figures.

        Args:
            filename (str): full path and filename of the resulting gif (e.g. "sample.gif")
            fps (float, optional): Frames per second. Defaults to 30.0.
        """

        self.path = "/".join(filename.split(".")[0].split("/")[:-1])
        self.path = self.path if self.path else os.getcwd()
        self.name = filename.split(".")[0].split("/")[-1]
        self.fps = fps
        self.temp_dir = "gifmaker_cache"
        self.i = 0

    def add(self, fig):
        fig.savefig(
            f"{self.temp_dir}/{self.name}" + str(self.i).zfill(4) + ".png",
            bbox_inches="tight",
        )
        plt.close()
        self.i += 1

    def save(self):

        fp_in = "{}/{}*.png".format(self.temp_dir, self.name)
        fp_out = "{}/{}.gif".format(self.path, self.name)

        imgs = [Image.open(f) for f in sorted(glob.glob(fp_in))]

        imgs[0].save(
            fp=fp_out,
            format="GIF",
            append_images=imgs[1:],
            save_all=True,
            duration=1000 / self.fps,
            loop=0,
        )

    def show(self):
        fp_out = "{}/{}.gif".format(self.path, self.name)
        return display.Image(filename=fp_out)

    def __enter__(self):
        if Path(self.temp_dir).exists():
            shutil.rmtree(self.temp_dir)
        os.mkdir(self.temp_dir)
        return self

    def __exit__(self, *kw):
        self.save()
        shutil.rmtree(self.temp_dir)


from setuptools import setup, find_packages

setup(
    name="lipstick",
    version="0.0.5",
    author="Mohammad Bashiri",
    author_email="mohammadbashiri93@gmail.com",
    description="A collection of plotting functions.",
    url="https://github.com/mohammadbashiri/lipstick",
    packages=find_packages(),
    install_requires=["numpy", "matplotlib>=3.3.0", "pillow>=7.2.0"],
)



import numpy as np
import matplotlib.pyplot as plt


def scatter_diag(x, y, fig_kws=None, scatter_kws=None, diag_kws=None):
    """
    Scatter plot with diagonal line.

    Returns:
        tuple: fig, ax
    """

    fig_kws = fig_kws if fig_kws is not None else {}

    if fig_kws is not None:
        pass
    else:
        fig_kws = {"figsize": (5.5, 5.5)}

    fig, ax = plt.subplots(**fig_kws)

    # the scatter plot:
    scatter_kws = (
        scatter_kws
        if scatter_kws is not None
        else {"c": "k", "ec": "w", "linewidth": 0.5}
    )
    ax.scatter(x, y, **scatter_kws)
    ax.set_aspect(1.0)

    min_lim = min(min(x), min(y))
    max_lim = max(max(x), max(y))

    diag_kws = diag_kws if diag_kws is not None else {"c": ".4", "ls": "--"}
    ax.plot([min_lim, max_lim], [min_lim, max_lim], max_lim, **diag_kws)

    return fig, ax


def scatter_hist(
    x,
    y,
    diag_on=False,
    x_hist_height=1.0,
    y_hist_width=1.0,
    fig_kws=None,
    scatter_kws=None,
    diag_kws=None,
    hist_kws=None,
    binwidth=0.1,
):
    """
    Scatter plot with histograms of x and y data.

    Returns:
        tuple: fig, (scatter_axis, hist_x_axis, hist_y_axis)
    """

    from mpl_toolkits.axes_grid1 import make_axes_locatable

    fig_kws = fig_kws if fig_kws is not None else {}

    if fig_kws is not None:
        pass
    else:
        fig_kws = {"figsize": (5.5, 5.5)}

    fig, axScatter = plt.subplots(**fig_kws)

    # the scatter plot:
    scatter_kws = scatter_kws if scatter_kws is not None else {"s": 10, "color": "k"}
    axScatter.scatter(x, y, **scatter_kws)
    axScatter.set_aspect(1.0)

    if diag_on:
        min_lim = min(min(x), min(y))
        max_lim = max(max(x), max(y))

        diag_kws = diag_kws if diag_kws is not None else {"c": ".4", "ls": "--"}
        axScatter.plot([min_lim, max_lim], [min_lim, max_lim], max_lim, **diag_kws)

    # create new axes on the right and on the top of the current axes
    # The first argument of the new_vertical(new_horizontal) method is
    # the height (width) of the axes to be created in inches.
    divider = make_axes_locatable(axScatter)
    axHistx = divider.append_axes("top", x_hist_height, pad=0.1, sharex=axScatter)
    axHisty = divider.append_axes("right", y_hist_width, pad=0.1, sharey=axScatter)

    # make some labels invisible
    axHistx.xaxis.set_tick_params(labelbottom=False)
    axHisty.yaxis.set_tick_params(labelleft=False)

    if hist_kws is not None:
        if "bins" not in hist_kws.keys():
            min_lim = min(min(x), min(y))
            max_lim = max(max(x), max(y))
            drange = max_lim - min_lim
            bins = np.arange(
                min_lim - 0.1 * drange, max_lim + 0.1 * drange + binwidth, binwidth
            )
    else:
        min_lim = min(min(x), min(y))
        max_lim = max(max(x), max(y))
        drange = max_lim - min_lim
        bins = np.arange(
            min_lim - 0.1 * drange, max_lim + 0.1 * drange + binwidth, binwidth
        )
        hist_kws = {"color": "k"}

        print(bins[:5])

    axHistx.hist(x, bins=bins, **hist_kws)
    axHisty.hist(y, bins=bins, orientation="horizontal", **hist_kws)

    return fig, (axScatter, axHistx, axHisty)